#include <stdio.h>
#include <math.h>
#include "cpu-helper.h"
#include "moments-loader.h"
#include "moments-kernel.h"

//--------------------------------------------------------------------------
// Random floats between lower and upper (inclusive).  Note that we
// construct the number using double precision.
void rand_float_vector(float *x, const uint n, 
		       const double lower, const double upper) {

  double gap = upper - lower;
  
  for(uint i = 0; i < n; i++) {
    double unit_uniform = (double)rand() / (double)RAND_MAX;
    x[i] = (float)(gap * unit_uniform + lower);
  }
}

//----------------------------------------------------------------------
void moments_protocol(const uint n, const uint max_moment,
		      const double lower, const double upper) {

  // Place to store the data.
  float *x;
  double *mom_gpu;

  // Allocate the vectors.
  NewArrayCPU(float, n, x);
  NewArrayCPU(double, max_moment, mom_gpu);

  // Fill the initial vector.
  rand_float_vector(x, n, lower, upper);

  // Pointers to the data on the GPU
  float *d_x;
  double *d_moments;
  // Allocate memory on the GPU and copy over input data.
  data_to_gpu(x, n, max_moment, &d_x, &d_moments);
  // Compute the moments.
  compute_all_gpu(d_x, n, max_moment, d_moments);
  // Make sure the GPU calculations are complete.
  after_run_gpu();
  // Copy back the results from the GPU and deallocate the memory on the GPU.
  data_from_gpu(mom_gpu, max_moment, d_x, d_moments);

  // Show the results.
  for(uint j = 0; j < max_moment; j++) {
    printf("\nMoment %d: %f", j+1, mom_gpu[j]);
  }

  // Free up all the CPU memory resources.
  free(x);
  free(mom_gpu);
}


//----------------------------------------------------------------------
int main(int argc, char **argv) {

  uint n_default = 1000000, max_moment_default = 4;
  double lower_default = 0.0, upper_default = 1.0; 

  printf("Computing %d moments for %d samples in range [ %f, %f ]",
	 max_moment_default, n_default, lower_default, upper_default);
  moments_protocol(n_default, max_moment_default, 
		   lower_default, upper_default);
  printf("\n\n");
}
